use std::{ops::Deref, rc::Rc};

use itertools::Itertools;
use yew::{function_component, html, AttrValue, Html, UseStateHandle};

use super::{close_editor, run_script, EditorViewDetailProps, EditorViewState};

fn show_create_dialog(state: &UseStateHandle<Rc<EditorViewState>>) {
    let mut new_state: EditorViewState = (*state.deref().deref()).clone();
    new_state.show_create_dialog = true;
    state.set(new_state.into());
}

#[function_component(EditorNav)]
pub(super) fn editor_nav(props: &EditorViewDetailProps) -> Html {
    let global_memo = props.global_memo.clone();
    let global_layout = props.global_layout.clone();
    let frame = props.frame.clone();
    let current_script = props.editor_state.open_file.clone();

    let set_err_state = props.editor_state.clone();
    let set_err = Rc::new(move |msg: Option<&str>| {
        let mut new_state = (*set_err_state.as_ref()).clone();
        new_state.error_msg = msg.map(|v| AttrValue::from(String::from(v)));
        set_err_state.set(new_state.into());
    });
    let editor_state = props.editor_state.clone();
    html! {
        <div class="editornav">
          <nav class="navbar navbar-expand-lg bg-body-tertiary">
            <button class="btn" aria-label="Run" title="Run"
              onclick={move |_ev| run_script(&current_script, &global_memo, set_err.clone())}>
              <i class="bi bi-file-earmark-play"></i></button>
            <button class="btn" aria-label="New file" title="New file"
              onclick={move |_ev| show_create_dialog(&editor_state) }
            ><i class="bi bi-file-earmark-plus"></i></button>
            <div class="flex-fill"/>
            <button class="btn" onclick={move |_ev| close_editor(&frame, &global_layout)} aria-label="Close Editor" title="Close editor"><i class="bi bi-arrow-return-left"></i></button>
          </nav>
          <ul class="p-2 list-group">
        {props
         .editor_state
         .available_files
         .iter()
         .map(|f| {
             let mut classes = vec!["list-group-item", "pe-auto"];
             let mut aria_current = None;
             if *f == props.editor_state.open_file {
                 aria_current = Some("true");
                 classes.push("active");
             }
             html! { <li aria-current={aria_current} tabindex={0}
                      class={classes.iter().join(" ")}
                      style="cursor: pointer"
                      >{f}</li>}
         })
         .collect::<Vec<Html>>()
        }
          </ul>
        </div>
    }
}
